// VARIANT.CPP -- Simple class to manage casts between different data types.
// 	(Implementation)
#include <stdlib.h>
#include "stdafx.h"
#include "variant.h"

ENUM_RFX_TYPE MapSQLTypeToEnumRFXType(int nType)
{        
	ENUM_RFX_TYPE eType = eUnassigned;

	switch(nType)
	{
		case SQL_BIT:
   		eType = eBit;
         break;      
                      
		case SQL_TINYINT:
   		eType = eByte;
			break;
                      
		case SQL_SMALLINT:
			eType = eInt;
         break;
         
      case SQL_INTEGER:
   		eType = eLong;
         break;
                                                                 
   	case SQL_BINARY:
   	case SQL_VARBINARY:
   		// case SQL_LONGVARBINARY:  See RFX_LongBinary below...                                                             
   		eType = eBinary;
         break;   
         
      case SQL_LONGVARBINARY:
      case SQL_LONGVARCHAR:   
			eType = eLongBinary;
         break;
                     
		case SQL_REAL:                     
			eType = eSingle;
      	break;
      	    
      case SQL_DOUBLE:
			eType = eDouble;
      	break;
      	
      case SQL_DATE:
      case SQL_TIME:
      case SQL_TIMESTAMP:
			eType = eDate;
         break;
              
      // case SQL_LONGVARCHAR: See RFX_LongBinary() instead.
      case SQL_CHAR:   
      case SQL_VARCHAR:
      case SQL_DECIMAL:
      case SQL_NUMERIC:
			eType = eText;
			break;
	}  
	return eType;
}			                     

ENUM_CONVERT_RULE MapStringToEnumConvType(CString& szRule)
{  
	ENUM_CONVERT_RULE	eRule;
              
   if(szRule == "Copy")
   	eRule = eCopy;           
	else if(szRule == "BlankValue")
		eRule = eBlankValue;
	else if(szRule == "StuffValue")
		eRule = eStuffValue;
	else if(szRule == "LeftTrim")
		eRule = eLeftTrim;
	else if(szRule == "RightTrim")
		eRule = eRightTrim;
	else if(szRule == "ToNumber")
		eRule = eToNumber;
	else if(szRule == "ToString")
		eRule = eToString;
	else if(szRule == "YesNoToLogical")
		eRule = eYNToLogical;
	else if(szRule == "LogicalToLogical")
		eRule = eLogicalToYN;
	else
		eRule = eUnknown;		 
		
	return eRule;
}		
                   
LPVOID CVariant::GetDataPtr(void)
{
	switch(eType)
	{        
		case eBit:
		case eByte:
		case eInt:
			return (LPVOID)&IntDatum;
		case eLong:
			return (LPVOID)&LongDatum;
		case eBinary:
			return (LPVOID)&Binary;
		case eLongBinary:
			return (LPVOID)&LongBinary;
		case eSingle:
			return (LPVOID)&SingleDatum;
		case eDouble:
			return (LPVOID)&DoubleDatum;			
		case eDate:
			return (LPVOID)&DateDatum;								
		case eText:                    
			return (LPVOID)&StringDatum;	
	   default: // eUnassigned.
#ifdef _DEBUG
			MessageBox(NULL, "Returning NULL pointer for eUnassigned data item in CVariant::GetDataPtr()", "Warning", MB_ICONEXCLAMATION); 
#endif	   
	   	return NULL;
	}
}

// Constructors
// Default Constructor
CVariant::CVariant(void)
{
	Init();
}
                        
// Copy Contructor
CVariant::CVariant(const CVariant& oldVar)
{
	Init();
   Assign(oldVar);
}                       
                        
CVariant::CVariant(const BOOL NewData, int nDummy)
{
	Init();
	eType = eBit;
	IntDatum = NewData;
}	

CVariant::CVariant(const BYTE NewData)
{
	Init();
	eType = eByte;
	IntDatum = NewData;
}	

CVariant::CVariant(const short int NewData)
{         
	Init();
	eType = eInt;
	IntDatum = NewData;
}	

CVariant::CVariant(const long NewData)
{
	Init();
	eType = eLong;
	LongDatum = NewData;
}	

CVariant::CVariant(CByteArray *NewData)
{             
	Init();
	eType = eBinary;
	CopyCByteArray(Binary, *NewData);   
}	

CVariant::CVariant(const CLongBinary *NewData)		
{
	Init();
	eType = eLongBinary;
	LongBinary.m_dwDataLength = NewData->m_dwDataLength;    // We can't delete this!!!
	LongBinary.m_hData = NewData->m_hData;
}
					
CVariant::CVariant(const float NewData)
{
	Init();
	eType = eSingle;
	SingleDatum = NewData;
}
	
CVariant::CVariant(const double NewData)
{ 
	Init();
	eType = eDouble;
	DoubleDatum = NewData;
}	

CVariant::CVariant(const CTime& NewData)
{
  	Init();
	eType = eDate;
	DateDatum = NewData;
}

CVariant::CVariant(const CString& NewData)
{
	Init();
	eType = eText;
	StringDatum = NewData;
}

CVariant::~CVariant()
{     
	CleanUp();
}
                                     
void CVariant::CleanUp(void)                                     
{
	// Clean-up any allocated objects here.
}

void CVariant::Init(void)
{
   eType = eUnassigned;

	IntDatum = 0;   		// For eBit, eByte, eInt, too.
	LongDatum = 0L; 		
	Binary.SetSize(0);
	LongBinary.m_dwDataLength = 0L;		
	LongBinary.m_hData = NULL;
   SingleDatum = 0.0; 
   DoubleDatum = 0.0;
	DateDatum = NULL;
   StringDatum = ""; 
}

CVariant& CVariant::Assign(const CVariant& oldVar)
{
	if(&oldVar != this)  // Avoid self-assignment.
	{
		Clear();
	   eType = oldVar.eType;

		IntDatum = oldVar.IntDatum;   		// For eBit, eByte, eInt, too.
		LongDatum = oldVar.LongDatum; 		
		CopyCByteArray(Binary, oldVar.Binary);
		CopyCLongBinary(LongBinary, oldVar.LongBinary);		
   	SingleDatum = oldVar.SingleDatum; 
   	DoubleDatum = oldVar.DoubleDatum;
		DateDatum = oldVar.DateDatum;
   	StringDatum = oldVar.StringDatum; 
   }
   return *this;
}
                       
BOOL CVariant::SetNumberFamilyValue(const double& NumVal, ENUM_RFX_TYPE eThisType)
{         
	int rc = TRUE;
#ifdef _DEBUG
	ASSERT(IsNumberFamily(eThisType));      	
#endif	                              
	switch(eThisType)
	{
		case eBit:
		case eByte:
		case eInt:
			IntDatum = (int)NumVal;
			break;
		case eLong:
			LongDatum = (long)NumVal;
			break;
		case eSingle:
			SingleDatum = (float)NumVal;
			break;
		case eDouble:
			DoubleDatum = (double)NumVal;	
			break;	
		default:   
			rc = FALSE;	
	}	
	return rc;
}

BOOL WriteBinaryToString(CString *pThisStr, const CByteArray *pThisBinary)
{
	long lBytes = pThisBinary->GetSize();
	
	*pThisStr = "";
	if(lBytes > MAXINT)                                
   	return FALSE;

   BYTE ch[2];
   ch[1] = '\0';         
	for(long i = 0; i < lBytes; i++)
	{
		ch[0] = (char)pThisBinary->GetAt((int)i); 
		(*pThisStr) += (LPSTR)ch;
	}	
	return TRUE;                                            
}

void CopyCLongBinary(CLongBinary& Targ, const CLongBinary& Src)
{                     
	// KNOWN BUG!!! -- CLongBinary values are not successfully converted.  (The MFC documentation
	// says that you can manipulate the m_hData "like any HGLOBAL value."  This should
	// include assigning a new handle...  Creating a new global data item [with GlobalAlloc()/
	// GlobalLock()] and writing test data still does not work.
	
	// (I'd appreciate anyone's solution on this...  -RD)
	Targ.m_dwDataLength = Src.m_dwDataLength;
	Targ.m_hData = Src.m_hData;
}		
                                            
                                            
                                            
// Default decimal places for converted floating point values.
#define MAX_DECIMAL_COUNT 2

BOOL CVariant::ConvertData(CVariant& oldVar, ENUM_CONVERT_RULE eRule, CString& szConvData)
{  
	// Add your own processing here for custom convert rules.
	ENUM_RFX_TYPE ePreserveMyType = eType;
	// Clear data.
	Clear();      
	eType = ePreserveMyType;	
	           
	double fStuffVal;
	CString szStuffVal;
	
	CString szVal;
	char szTemp[50];
		           
	int nSrcLen, nTrimLen;	             
	int nFirst, nLast;            
	    
	int bSrcIsNum, bSrcIsText;
	int bTargIsNum, bTargIsText;    
	     	                  
	int rc;	     	                  

	int Dec, Sign;                     
	     	                  
	CString szData;
	CString szDec;
	int nDec;
		
	switch(eRule)
	{	
		case eCopy:
			// Here's what we know how to do--
			// Copy number family to number family.
			// Copy text family to text family.
			// Copy date/time/timestamp to same.					
			// Copy long binaries to same.
		   if(IsNumberFamily(oldVar.eType) && IsNumberFamily(eType))
		   {
		   	switch(oldVar.eType)
   			{
					case eBit:
					case eByte:
					case eInt:      
						IntDatum = oldVar.IntDatum;
						break;
					case eLong:
						LongDatum = oldVar.LongDatum;
						break;
					case eSingle:    
						SingleDatum = oldVar.SingleDatum;
						break;
					case eDouble:
						DoubleDatum = oldVar.DoubleDatum;
						break;
					default:   
						return FALSE;	
				}
				return TRUE;
			}
		   else if(IsTextFamily(oldVar.eType) && IsTextFamily(eType))
		   {  
		     	if(oldVar.eType == eText && oldVar.eType == eText)
		   		StringDatum = oldVar.StringDatum;
		   	else if(oldVar.eType == eBinary && oldVar.eType == eBinary)
		   		CopyCByteArray(Binary, oldVar.Binary);
		   	else if(oldVar.eType == eBinary && oldVar.eType == eText)
				{                  
					rc = WriteBinaryToString(&StringDatum, &oldVar.Binary);
					if(!rc)
						return FALSE;
				}			   		
		      else  // Then it must be string to memo.
				{
					Binary.SetSize(0);
					int nBytes = oldVar.StringDatum.GetLength();
					LPSTR pBuff = (LPSTR)oldVar.StringDatum.GetBuffer(nBytes + 1);
					for(int i = 0; i < nBytes; i++)
						Binary.SetAtGrow(i, (BYTE)pBuff[i]);
			   	return TRUE;
				}
				return TRUE;		   
		   }                    
		   else if(oldVar.eType == eDate && eType == eDate)        
		   {
		   	DateDatum = oldVar.DateDatum;
		      return TRUE;
		   }			   
		   else if(oldVar.eType == eLongBinary && eType == eLongBinary)        
		   {                             
		   	// Copy long binary info.
		   	CopyCLongBinary(LongBinary, oldVar.LongBinary);
		   	return TRUE;
		   }              
		   else                   
		   	// Couldn't copy!!!
		   	return FALSE;
		   
		case eBlankValue:
	      break;		
		case eStuffValue:
			if(IsNumberFamily(eType))
			{
				fStuffVal = atof(szConvData.GetBuffer(256));
				return SetNumberFamilyValue(fStuffVal, eType);
			}	
			else if (eType == eText)
			{
				StringDatum = szConvData;			
				return TRUE;
			}	                          
			else if(eType == eBinary)
			{
				// Lets us stuff a memo. field
				Binary.SetSize(0);
				int nBytes = szConvData.GetLength();
				LPSTR pBuff = szConvData.GetBuffer(nBytes + 1);
				for(int i = 0; i < nBytes; i++)
					Binary.SetAtGrow(i, (BYTE)pBuff[i]);
			   return TRUE;
			}
			else
				return FALSE;  // Unknown!	
			break;
					
		case eLeftTrim:
		case eRightTrim:
			if(IsTextFamily(eType))
			{              
				nTrimLen = atoi(szConvData.GetBuffer(256));
				if(eType == eText)
				{
					// For strings.
					nSrcLen = oldVar.StringDatum.GetLength();
					if(nSrcLen > nTrimLen)
					{
					   if(eRule == eLeftTrim)                   
					   	StringDatum = oldVar.StringDatum.Left(nTrimLen);
					   else // eRightTrim	                                            
					   	StringDatum = oldVar.StringDatum.Right(nTrimLen);
					}
					else
						StringDatum = oldVar.StringDatum;
					return TRUE;
				}
				else  // i.e., (eType == eBinary)                  
				{
					Binary.SetSize(0);
				                          
				   nSrcLen = oldVar.Binary.GetSize();
				        
				   if(nSrcLen > nTrimLen)
				   {
				   	// Then trim...
				   	if(eRule == eLeftTrim)
						{
				   		nFirst = 0;
				   		nLast = nTrimLen - 1;	
				   	}	
				   	else // eRightTrim	     
				   	{
				   	   nFirst = nSrcLen - nTrimLen;
				   	   nLast = nSrcLen - 1;
				   	} 
				   }
				   else
				   {
				   	// Copy whole thing.
						nFirst = 0;
				   	nLast = nSrcLen - 1;                      
		         }                                                     
					for(int i = nFirst; i < nLast; i++)
						Binary.SetAtGrow(i - nFirst, oldVar.Binary.GetAt(i));	
				}			
			}
			else
				return FALSE;		
		   break;
		   
		case eToString:
			bSrcIsNum = IsNumberFamily(oldVar.eType);
			bTargIsText = IsTextFamily(eType);			
			if(!bSrcIsNum || !bTargIsText)
				return FALSE;
   		switch(oldVar.eType)
   		{
				case eBit:
				case eByte:
				case eInt:      
					itoa(oldVar.IntDatum, szData.GetBuffer(10), 10);
					StringDatum = szData;
					break;
				case eLong:
					ltoa(oldVar.LongDatum, szData.GetBuffer(50), 10);
					StringDatum = szData;
					break;
				case eSingle:    
				case eDouble:            
					if(oldVar.eType == eSingle)
						lstrcpy(szTemp, _fcvt((double)oldVar.SingleDatum, MAX_DECIMAL_COUNT, &Dec, &Sign));
					else // eDouble
						lstrcpy(szTemp, _fcvt(oldVar.DoubleDatum, MAX_DECIMAL_COUNT, &Dec, &Sign));
					szData = szTemp;
					if(Sign)
						StringDatum = "-";
					else
						StringDatum = "";	
					StringDatum += szData.Left(Dec);

					szDec = szData.Mid(Dec);
					nDec = atoi(szDec.GetBuffer(50));
					
					if(nDec != 0)
					{
						itoa(nDec, szTemp, 10);
						szDec = szTemp;
						StringDatum += ".";
						StringDatum += szDec.Mid(0, MAX_DECIMAL_COUNT);
					}
					break;	
				default:   
					return FALSE;	
			}	
	      break;
			
		case eToNumber:
			bSrcIsText = IsTextFamily(oldVar.eType);
			bTargIsNum = IsNumberFamily(eType);			
			if(!bSrcIsText || !bTargIsNum)
				return FALSE;
         
         if(oldVar.eType == eBinary)
         {
         	rc = WriteBinaryToString(&szData, &oldVar.Binary);
            if(!rc)
            	return FALSE;
         }
         else   		                         
          	szData = oldVar.StringDatum;	
                                     
		   switch(eType)
   		{
				case eBit:
				case eByte:
				case eInt:      
					IntDatum = atoi(szData);
					break;
				case eLong:
					LongDatum = atol(szData);
					break;
				case eSingle:               
					SingleDatum = (float)atof(szData);
					break;
				case eDouble:
					DoubleDatum = (double)atof(szData);
					break;
				default:   
					return FALSE;	
			}	
	      break;
		         
		case eYNToLogical:      
			bSrcIsText = IsTextFamily(oldVar.eType);
			rc = (eType == eBit || eType == eByte || eType == eInt);
			if(!bSrcIsText || !rc)
				return FALSE;
            
			// Otherwise, make the conversion.
			if(oldVar.eType == eBinary)
			{
  	   	  	rc = WriteBinaryToString(&szData, &oldVar.Binary);
   	      if(!rc)
           		return FALSE;
  			}								
			else // oldVar.eType = eText
				szData = oldVar.StringDatum;			             
			             
			szData = szData.Left(1);
			
			if(szData == "Y")
				IntDatum = 1;
			else
				IntDatum = 0;	
			break;
					
		case eLogicalToYN:
			rc = oldVar.eType == eBit || oldVar.eType == eByte ||
				  oldVar.eType == eInt;
			bTargIsText = IsTextFamily(eType);
			
			if(!rc || !bTargIsText)
				return FALSE;
            
			// Otherwise, make the conversion.
			if(eType == eBinary)
			{  
      		if(IntDatum != 0)
					szData = "Y";			
				else
				   szData = "N";
			                
				Binary.SetSize(1);
      		if(IntDatum != 0)
					Binary.SetAt(0, 'Y');
				else
					Binary.SetAt(0, 'N');
			}		            
			else
			{
      		if(IntDatum != 0)
					szData = "Y";			
				else
				   szData = "N";
			}
     		return TRUE;
         
   	default:        
#ifdef _DEBUG
			MessageBox(NULL, "Unexpected type in CVariant::ConvertData()", "Unexpected Error", MB_ICONSTOP);
#endif   	
   		return FALSE;
   }
   return TRUE;
}
