// Simple class to manage casts between different data types.
#if !defined(_VARIANT_H)
	#define _VARIANT_H   
                                                             
enum ENUM_RFX_TYPE {eUnassigned,
					eBit,
					eByte,
					eInt,
					eLong,
					eBinary,
					eLongBinary,
					eSingle,
					eDouble,
					eDate,
					eText };
		
inline IsIntFamily(ENUM_RFX_TYPE eTest)
{
	if(eTest == eBit || eTest == eByte || eTest == eInt)
		return TRUE;
	else
		return FALSE;		
}

inline IsNumberFamily(ENUM_RFX_TYPE eTest)
{
	if(eTest == eBit || eTest == eByte || eTest == eInt ||
		eTest == eLong || eTest == eSingle || eTest == eDouble)
		return TRUE;
	else
		return FALSE;		
}

inline IsTextFamily(ENUM_RFX_TYPE eTest)
{           
	// Text and memo.--but not 'long binary' memo. fields.
	if(eTest == eText || eTest == eBinary)
		return TRUE;
	else
		return FALSE;		
}
							
enum ENUM_CONVERT_RULE { eUnknown,
									eCopy,
									eBlankValue,
									eStuffValue,
									eLeftTrim,
									eRightTrim,
									eToString,
									eToNumber,
									eYNToLogical,
									eLogicalToYN };

ENUM_RFX_TYPE MapSQLTypeToEnumRFXType(int nType);
               
ENUM_CONVERT_RULE MapStringToEnumConvType(CString& szRule);

// Helper function since there are no copy constructors
// for many MFC classes.
                                       
inline void CopyCByteArray(CByteArray& Targ, const CByteArray& Src)
{                      
	int nSize = Src.GetSize();

	Targ.SetSize(nSize);
	
	for(int i = 0; i < nSize; i++)
		Targ.SetAt(i, Src.GetAt(i));	
}
                              
void CopyCLongBinary(CLongBinary& Targ, const CLongBinary& Src);
                                         
BOOL WriteBinaryToString(CString *pThisStr, const CByteArray *pThisBinary);
                                         
// Simple class to manage casts between different data types --
// 	(Declaration)
                           
class CVariant
{
public:   
   ENUM_RFX_TYPE eType;

	int IntDatum;   		// For eBit, eByte, eInt, too.
	long LongDatum; 		
	CByteArray Binary;
	CLongBinary LongBinary;		
   float SingleDatum; 
   double DoubleDatum;
	CTime DateDatum;
   CString StringDatum;      // For SQL_CHAR, SQL_VARCHAR, SQL_DECIMAL, SQL_NUMERIC
          
   // Get pointer to our data, according to its type.                  
   LPVOID GetDataPtr(void);
   
   // Constructors
   // Default
   CVariant(void);
               
	CVariant(const CVariant& oldVar);
	CVariant(const BOOL NewData, int nDummy);
	CVariant(const BYTE NewData);
	CVariant(const short int NewData);
	CVariant(const long NewData);
	CVariant(CByteArray *NewData);
	CVariant(const CLongBinary *NewData);		
   CVariant(const float NewData);
   CVariant(const double NewData);
	CVariant(const CTime& NewData);
	CVariant(const CString& NewData);
   
   ~CVariant();
   
   void CleanUp();
                   
	void Init(void);      
	void Cleanup(void);
	void Clear() { CleanUp(); Init(); }
		
	CVariant& Assign(const CVariant& oldVar);
	CVariant& operator=(const CVariant& oldVar) { return Assign(oldVar); }           

	BOOL SetNumberFamilyValue(const double& NumVal, ENUM_RFX_TYPE eThisType);
	
	BOOL ConvertData(CVariant& oldVar, ENUM_CONVERT_RULE eRule, CString& szConvData);
	BOOL ConvertData(CVariant& oldVar, CString& szRule, CString& szConvData) { 
			ENUM_CONVERT_RULE eRuleType = MapStringToEnumConvType(szRule);
			return ConvertData(oldVar, eRuleType, szConvData);
		}		
};
                          
typedef struct _COLUMNINFO
{         
	int nSQLDataType;  
	int Len;
	
	CString szFieldName;
	
	CVariant ColData;	
} COLUMNINFO;                         
                          

typedef struct _BOUNDCOLUMNSINFO
{
	int nSourceCol;	
   int nTargetCol;
   ENUM_CONVERT_RULE eConvRule;
   CString szConvData;
} BOUNDCOLUMNSINFO;
               
#endif               
                      
                      


